use std::{fmt, io, num::TryFromIntError};

#[derive(Debug)]
pub enum Error {
    Io(io::Error),
    Fmt(fmt::Error),
    IllegalChar(char),
    IntError(TryFromIntError),
    MissingChar,
}

impl fmt::Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Io(e) => write!(f, "{e}"),
            Self::Fmt(e) => write!(f, "{e}"),
            Self::IllegalChar(c) => write!(f, "Illegal Character ({c})"),
            Self::IntError(e) => write!(f, "Int Error: {e}"),
            Self::MissingChar => write!(f, "Missing character"),
        }
    }
}

impl From<io::Error> for Error {
    fn from(value: io::Error) -> Self {
        Self::Io(value)
    }
}

impl From<char> for Error {
    fn from(value: char) -> Self {
        Self::IllegalChar(value)
    }
}

impl From<fmt::Error> for Error {
    fn from(value: fmt::Error) -> Self {
        Self::Fmt(value)
    }
}

impl From<TryFromIntError> for Error {
    fn from(value: TryFromIntError) -> Self {
        Self::IntError(value)
    }
}

impl std::error::Error for Error {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Self::Fmt(e) => Some(e),
            Self::Io(e) => Some(e),
            Self::IntError(e) => Some(e),
            _ => None,
        }
    }
}
